@@ -1,5 +1,3 @@
-#!/usr/local/bin/ruby -w
-
 # tc_highline.rb
 #
 #  Created by James Edward Gray II on 2005-04-26.
@@ -62,6 +60,82 @@ class TestHighLine < Test::Unit::TestCase
     assert_raise(EOFError) { @terminal.ask("Any input left?  ") }
   end
   
+  def test_ask_string
+    name = "James Edward Gray II"
+    @input << name << "\n"
+    @input.rewind
+
+    assert_equal(name, @terminal.ask("What is your name?  ", String))
+
+    assert_raise(EOFError) { @terminal.ask("Any input left?  ", String) }
+  end
+
+  def test_indent
+    text = "Testing...\n"
+    @terminal.indent_level=1
+    @terminal.say(text)
+    assert_equal(' '*3+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_level=3
+    @terminal.say(text)
+    assert_equal(' '*9+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_level=0
+    @terminal.indent_size=5
+    @terminal.indent(2, text)
+    assert_equal(' '*10+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_size=4
+    @terminal.indent {
+        @terminal.say(text)
+    }
+    assert_equal(' '*4+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent_size=2
+    @terminal.indent(3) { |t|
+        t.say(text)
+    }
+    assert_equal(' '*6+text, @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent { |t|
+        t.indent {
+            t.indent {
+                t.indent { |tt|
+                    tt.say(text)
+                }
+            }
+        }
+    }
+    assert_equal(' '*8+text, @output.string)
+
+    text = "Multi\nLine\nIndentation\n"
+    indent = ' '*4
+    @terminal.indent_level=2
+    @output.truncate(@output.rewind)
+    @terminal.say(text)
+    assert_equal("#{indent}Multi\n#{indent}Line\n#{indent}Indentation\n", @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.multi_indent = false
+    @terminal.say(text)
+    assert_equal("#{indent}Multi\nLine\nIndentation\n", @output.string)
+
+    @output.truncate(@output.rewind)
+    @terminal.indent(0, text, true)
+    assert_equal("#{indent}Multi\n#{indent}Line\n#{indent}Indentation\n", @output.string)
+  end
+  
+  def test_newline
+    @terminal.newline
+    @terminal.newline
+    assert_equal("\n\n", @output.string)
+  end
+
   def test_bug_fixes
     # auto-complete bug
     @input << "ruby\nRuby\n"
@@ -145,10 +219,10 @@ class TestHighLine < Test::Unit::TestCase
       @input << "\b\b"
       @input.rewind
       answer = @terminal.ask("Please enter your password: ") do |q| 
-          q.echo = "*" 
+        q.echo = "*" 
       end
       assert_equal("", answer)
-      assert_equal("Please enter your password: \n",@output.string)
+      assert_equal("Please enter your password: \n", @output.string)
   end
   
   def test_readline_on_non_echo_question_has_prompt
@@ -159,7 +233,7 @@ class TestHighLine < Test::Unit::TestCase
       q.echo = "*"
     end
     assert_equal("you can't see me", answer)
-    assert_equal("Please enter some hidden text: ****************\n",@output.string)
+    assert_equal("Please enter some hidden text: ****************\n", @output.string)
   end
   
   def test_character_reading
@@ -300,7 +374,19 @@ class TestHighLine < Test::Unit::TestCase
     assert_equal( "Are you sexually active?  |No Comment|  ",
                   @output.string )
   end
-  
+
+  def test_string_preservation
+    @input << "Maybe\nYes\n"
+    @input.rewind
+
+    my_string = "Is that your final answer? "
+
+    @terminal.ask(my_string) { |q| q.default = "Possibly" }
+    @terminal.ask(my_string) { |q| q.default = "Maybe" }
+
+    assert_equal("Is that your final answer? ", my_string)
+  end
+
   def test_empty
     @input << "\n"
     @input.rewind
@@ -336,8 +422,6 @@ class TestHighLine < Test::Unit::TestCase
       q.glob      = "*.rb"
     end
     assert_instance_of(File, file)
-    assert_equal("#!/usr/local/bin/ruby -w\n", file.gets)
-    assert_equal("\n", file.gets)
     assert_equal("# tc_highline.rb\n", file.gets)
     file.close
 
@@ -388,15 +472,72 @@ class TestHighLine < Test::Unit::TestCase
                   answers )
     assert_equal("Age:  Father's Age:  Wife's Age:  ", @output.string)
   end
- 
-  def test_lists 
+
+  def test_typing_verification
+    @input << "all work and no play makes jack a dull boy\n" * 3
+    @input.rewind
+
+    answer = @terminal.ask("How's work? ") do |q|
+      q.gather = 3
+      q.verify_match = true
+    end
+    assert_equal("all work and no play makes jack a dull boy", answer)
+
+    @input.truncate(@input.rewind)
+    @input << "all play and no work makes jack a mere toy\n"
+    @input << "all work and no play makes jack a dull boy\n" * 5
+    @input.rewind
+    @output.truncate(@output.rewind)
+
+    answer = @terminal.ask("How are things going? ") do |q|
+      q.gather = 3
+      q.verify_match = true
+      q.responses[:mismatch] = 'Typing mismatch!'
+      q.responses[:ask_on_error] = ''
+    end
+    assert_equal("all work and no play makes jack a dull boy", answer)
+
+    # now try using a hash for gather
+
+    @input.truncate(@input.rewind)
+    @input << "Password\nPassword\n"
+    @input.rewind
+    @output.truncate(@output.rewind)
+
+    answer = @terminal.ask("<%= @key %>: ") do |q|
+      q.verify_match = true
+      q.gather = {"Enter a password" => '', "Please type it again" => ''}
+    end
+    assert_equal("Password", answer)
+
+    @input.truncate(@input.rewind)
+    @input << "Password\nMistake\nPassword\nPassword\n"
+    @input.rewind
+    @output.truncate(@output.rewind)
+
+    answer = @terminal.ask("<%= @key %>: ") do |q|
+      q.verify_match = true
+      q.responses[:mismatch] = 'Typing mismatch!'
+      q.responses[:ask_on_error] = ''
+      q.gather = {"Enter a password" => '', "Please type it again" => ''}
+    end
+
+    assert_equal("Password", answer)
+    assert_equal( "Enter a password: " +
+                  "Please type it again: " +
+                  "Typing mismatch!\n" +
+                  "Enter a password: " +
+                  "Please type it again: ", @output.string )
+  end
+
+  def test_lists
     digits     = %w{Zero One Two Three Four Five Six Seven Eight Nine}
     erb_digits = digits.dup
     erb_digits[erb_digits.index("Five")] = "<%= color('Five', :blue) %%>"
-    
+
     @terminal.say("<%= list(#{digits.inspect}) %>")
     assert_equal(digits.map { |d| "#{d}\n" }.join, @output.string)
-    
+
     @output.truncate(@output.rewind)
 
     @terminal.say("<%= list(#{digits.inspect}, :inline) %>")
@@ -497,6 +638,14 @@ class TestHighLine < Test::Unit::TestCase
       assert_equal("", result)
     end
   end
+
+  def test_lists_with_nil_items
+    modes = [nil]
+    modes.each do |mode|
+      result = @terminal.list([nil], mode)
+      assert_equal("\n", result)
+    end
+  end
   
   def test_lists_with_one_item
     items = ['Zero']
@@ -756,6 +905,28 @@ class TestHighLine < Test::Unit::TestCase
 
     @terminal.say("This will not have a newline.  ")
     assert_equal("This will not have a newline.  ", @output.string)
+
+    @output.truncate(@output.rewind)
+    
+    @terminal.say("This will not\n end with a newline. ")
+    assert_equal("This will not\n end with a newline. ", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    @terminal.say("This will \nend with a newline.")
+    assert_equal("This will \nend with a newline.\n", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    colorized = @terminal.color("This will not have a newline. ", :green)
+    @terminal.say(colorized)
+    assert_equal("\e[32mThis will not have a newline. \e[0m", @output.string)
+
+    @output.truncate(@output.rewind)
+
+    colorized = @terminal.color("This will have a newline.", :green)
+    @terminal.say(colorized)
+    assert_equal("\e[32mThis will have a newline.\e[0m\n", @output.string)
   end
 
   def test_type_conversion
@@ -947,6 +1118,6 @@ class TestHighLine < Test::Unit::TestCase
     assert_not_nil(HighLine::VERSION)
     assert_instance_of(String, HighLine::VERSION)
     assert(HighLine::VERSION.frozen?)
-    assert_match(/\A\d\.\d\.\d\Z/, HighLine::VERSION)
+    assert_match(/\A\d+\.\d+\.\d+\Z/, HighLine::VERSION)
   end
 end
